/*
 * Copyright (c) 1999-2019 Hartmut Seichter
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
#ifndef PW_CORE_TIMER_HPP_
#define PW_CORE_TIMER_HPP_


#include <pw/core/globals.hpp>

#include <chrono>

namespace pw {

/**
 * @brief A simple timer
 */
class timer {
public:

    const static unsigned int UnitMicroSeconds = 1000000;
    const static unsigned int UnitMilliSeconds = 1000;
    const static unsigned int UnitSeconds = 1;

	typedef std::chrono::time_point<std::chrono::high_resolution_clock> tick_t;

	timer(); /// c'tor
	~timer(); /// d'tor

    void reset(); /// reset the timer

    /**
     * @brief elapsed time based on the scale
     * @param scale units the time will be reported
     * @return value of the time since last @see reset
     */
    double elapsed(unsigned int scale = UnitMilliSeconds) const;

    /**
     * @brief Now
     * @param scale
     * @return
     */
    static double now(unsigned int scale = UnitMicroSeconds);

protected:

	tick_t _start;
};

}


#endif
